#include <iostream>
#include <cstdlib>
#include <complex>
#include <cmath>

using namespace std;

const double pi=3.14159265358979311600;

extern void ApplyFilter(int,const int, const float*,float*,
			 const double*,const double*, int);
extern void GetFilterCoef(const int,const int,const float,const float,
			  double*,double*,const int);

void getFiltercoefLH(const int,const int,const float,double*,double*,const int);
void getFiltercoefBP(const int,const int,const float,const float,double*,double*);
void findpole(int,complex<double>*);
complex<double> coef(complex<double>*,int,int,int,int);
int binomial(int, int);
int sign(int);


// filter data
// N: filter order
// Ndata: number of data point to be filtered
// idata: input data array
// odata: output data array
// fcoefa & fcoefb: filter coefficients (denominator & numerator)
// mode: 0:lowpass 1:highpass 2:bandpass
void ApplyFilter(int N, const int Ndata, 
		 const float* idata, float* odata,
		 const double* fcoefa, const double* fcoefb, int mode)
{

  double* data = new double[Ndata];

  if (mode==2) N=N*2;

  for(int i = 0; i < Ndata; i++) {
    data[i] = fcoefb[0] * (double)idata[i];
    for(int j = 1; j <= N; j++){
      if( i >= j ) data[i] += ((fcoefb[j] * (double)idata[i-j]) - 
			       (fcoefa[j] * data[i-j]));
    }

    odata[i] = (float)data[i];

  }

  delete[] data;

  return;

}


// get filter coefs in the selected mode
// N: filter order
// sfreq: sampling frequency
// cfreqL: cut off frequency (lower edge)
// cfreqH: cut off frequency (upper edge)
// fcoefa & fcoefb: filter coefficients (denominator & numerator)
// mode: 0:lowpass 1:highpass 2:bandpass
void GetFilterCoef(const int N,const int sfreq,
		   const float cfreqL, const float cfreqH
		   , double* fcoefa, double* fcoefb,const int mode)
{

  switch (mode){
  case 0:
  case 1: 
    getFiltercoefLH(N,sfreq,cfreqL,fcoefa,fcoefb,mode);
    break;
  case 2: 
    getFiltercoefBP(N,sfreq,cfreqL,cfreqH,fcoefa,fcoefb);
    break;
  }

}

// compute low or high pass filter coefficients
void getFiltercoefLH(const int N,const int sfreq,const float cfreq,
		     double* fcoefa, double* fcoefb,const int mode)
{

  double e,a0;
  complex<double> temp(1.0,0.0);
  complex<double> root[N];
  complex<double> coef_denom[N];
  int i;

  e=tan(pi*cfreq/sfreq);
  if(mode==0) e=1/e;

  // filter coeff 1/b0
  findpole(N, root);
  for(i=0;i<N;i++) temp*=e-root[i];
  a0=temp.real();


  switch(mode){
  case 0:  //filter coef b for low pass
    for(i=0;i<=N;i++) fcoefb[i]=binomial(N,i)/a0;
    break;
  case 1:  //filter coef b for high pass
    for(i=0;i<=N;i++) fcoefb[i]=sign(i)*binomial(N,i)/a0;
    break;
  }

  for(i=0;i<N;i++){
    coef_denom[i]=(e+root[i])/(e-root[i]);
  }
  fcoefa[0]=1.0;
  switch (mode){
  case 0:  //filter coef a for low pass
    for(i=1;i<=N;i++) fcoefa[i]=sign(i)*coef(coef_denom,N,i,1,0).real();
    break;
  case 1:  //filter coef a for high pass
    for(i=1;i<=N;i++) fcoefa[i]=coef(coef_denom,N,i,1,0).real();
    break;
  }
   

  return;

}

// compute bandpass filter coefficients
void getFiltercoefBP(const int N,const int sfreq,const float cfreq0, const float cfreq1, double* fcoefa, double* fcoefb){

  double e0,e1,A;
  complex<double> temp(1.0,0.0);
  complex<double> root[2*N];
  complex<double> coef_denom[2*N];
  int i;

  e0=tan(pi*cfreq0/sfreq);
  e1=tan(pi*cfreq1/sfreq);

  // filter coeff 1/b0
  findpole(N, root);
  for(i=N-1;i>=0;i--){
    root[2*i+1]=(root[i]*(e1-e0)-
		 sqrt(root[i]*root[i]*(e1-e0)*(e1-e0)-
		      4*e1*e0))/2.0;
    root[2*i]=(root[i]*(e1-e0)+
	       sqrt(root[i]*root[i]*(e1-e0)*(e1-e0)-
		    4*e1*e0))/2.0;
  }
  for(i=0;i<2*N;i++) temp *= 1.0 - root[i];
  A=pow(e1-e0,N)/temp.real();

  for(i=0;i<=N;i++) fcoefb[2*i]=sign(i)*binomial(N,i)*A;
  for(i=0;i<N;i++) fcoefb[2*i+1]=0;
  for(i=0;i<2*N;i++) coef_denom[i] = (1.0 + root[i]) / (1.0 - root[i]);
  
  fcoefa[0]=1.0;

  //filter coef a for high pass
  for(i=1;i<=2*N;i++) fcoefa[i]=sign(i)*coef(coef_denom,2*N,i,1,0).real();

  return;

}


void findpole(int N, complex<double>* root){

  int j=0,i;

  if(N%2){
    for(i=N/2+1;i<3*N/2+1;i++) {
      root[j++]=complex<double>(cos(i*pi/N),sin(i*pi/N));
    }
  }else {
    for(i=N/2;i<3*N/2;i++) {
      root[j++]=complex<double>(cos((i+0.5)*pi/N),sin((i+0.5)*pi/N));
    }
  }

  return;

}

complex<double> coef(complex<double>* a, int N, int K, int count, int j){
  
  complex<double> sum=0;
  int i;

  if(!K) sum=0;
  else if(count==K){
    for(i=j;i<N-K+count;i++) sum=sum+a[i];
  }else{
    for(i=j;i<N-K+count;i++) sum=sum+a[i]*coef(a,N,K,count+1,i+1);
  }

  return sum;

}

int binomial(int N, int K){

  int prod=1,i;

  for(i=0;i<N-K;i++) prod=prod*(N-i)/(i+1);

  return prod;

}

int sign(int i){
  return (i%2)?-1:1;
}
