#ifndef magGlitch_HH
#define magGlitch_HH

#include <fstream>

#include "DatEnv.hh"
#include "TSeries.hh"
#include "Histogram1.hh"

#define PIDCVSHDR "$Header: /ldcg_server/common/repository_gds/gds/Monitors/magGlitch/magGlitch.hh,v 1.3 2005/01/31 19:51:25 jzweizig Exp $"
#define PIDTITLE "Magnetic Field Glitch Monitor"
#include "ProcIdent.hh"
#include "TrigClient.hh"
#include "TrigRslt.hh"
#include "MonServer.hh"

#include "magChannelStat.hh"

/** @name magGlitch
  * magGlitch checks magnetometer channels
  * for glitches within relatively broadband noise.  Due to the many peaks in
  * the magnetometer spectra which tend to dominate the band-limited power,
  * bands are selected which include as few peaks as possible.  Glitches which
  * occur less than .1 seconds apart are considered the same glitch.\\
  * {\bf Usage}: #magGlitch [<conf_file>]#
  *
  * {\large {\bf Output}}\\
  * Generally, no output is rewritten unless the monitor is started more than
  * once in the same day.  Instead, old log filenames and data filenames get
  * the current date appended to them.
  *
  * {\em Log File}-- The log file is usually located in the DMTHTMLOUT
  * directory for magGlitch.  On the web this can usually be viewed by clicking
  * on the magGlitch link from the DMT Monitor Observer web page.  At LHO, this
  * is \URL{http://blue.ligo-wa.caltech.edu/gds/monitor_reports/magGlitch/} and
  * at LLO, this is 
  * \URL{http://london.ligo-la.caltech.edu/gds/monitor_reports/magGlitch/}. Log
  * files report the specifications made in the configuration file, the number
  * of glitches found each Log Interval (both described below), and--- at the
  * end of the run--- a histogram for each band indicating the total number
  * of glitches found, binned by peak magnitude in units of $\sigma$.
  *
  * {\em Data File}-- The data file is usually located in the DMTOUTPUT
  * directory for magGlitch, and contains the records of all glitches
  * generated.  Data records are specified in the configuration file, described
  * below.  There are two types of records:
  * \begin{itemize}
  *   \item {\em glitch record}- A single-line record, separated by spaces,
  *   with the following information:\\
  *   #chan_name GPS_start GPS_end GPS_peak peak_mag ts_avg ts_sdev npts com#\\
  *   where #chan_name# is the name of the channel, #GPS_start/end/peak# are
  *   the start, end, and peak times of the glitch, #peak_mag# is the peak
  *   magnitude of the glitch (in $\sigma$), #ts_avg/sdev# are the current
  *   values for the running mean and standard deviation of the full time
  *   series, #npts# is the number of points exceeding the threshold during the
  *   glitch, and #com# is an optional comment field to describe the type of
  *   glitch which occurred.
  *   \item {\em histogram record}- A multi-line, formatted dump of the
  *   histograms generated during the previous Log Interval (described below).
  *   A histogram is generated for each band observed, binning the glitch peak
  *   magnitudes.
  * \end{itemize}
  *
  * {\large {\bf Configuration File}}-- The configuration file sets values for
  * run-time parameters and also provides the list of channels to run on.  A
  * sample configuration file follows:
  * \begin{verbatim}
 DataFilePath magE5H_200-300.dat
 LogFilePath magE5H_200-300.log
 LogInterval 3600
 DAQInterval 84600
 DecayTime 10.0
 TriggerON 0
 
 Channel H0:PEM-BSC1_MAG1X 2048 2.0 2.5 BandPass 5 185.0 235.0 185-235 End
 Channel H0:PEM-BSC1_MAG1Y 2048 2.0 2.5 BandPass 5 245.0 265.0 245-265 End
 Channel H0:PEM-BSC1_MAG1Z 2048 2.0 2.5 BandPass 5 280.0 293.0 280-293 End
 Channel H0:PEM-BSC1_MAGX 2048 2.0 2.5 BandPass 5 185.0 235.0 185-235 End
 Channel H0:PEM-BSC1_MAGY 2048 2.0 2.5 BandPass 5 245.0 265.0 245-265 End
 Channel H0:PEM-BSC1_MAGZ 2048 2.0 2.5 BandPass 5 280.0 293.0 280-293 End
 Channel H0:PEM-BSC10_MAGX 2048 2.0 2.5 BandPass 5 222.0 295.0 222-295 End
 Channel H0:PEM-BSC10_MAGY 2048 2.0 2.5 BandPass 5 222.0 295.0 222-295 End
 Channel H0:PEM-BSC10_MAGZ 2048 2.0 2.5 BandPass 5 222.0 295.0 222-295 End
 Channel H0:PEM-COIL_MAGZ  2048 2.0 2.5 BandPass 5 207.0 232.0 207-232 End
  \end{verbatim}
  * 
  * An explanation of the above parameters:
  * \begin{itemize}
  *   \item #DataFilePath   -# Data output filename
  *   \item #LogFilePath    -# Log output filename
  *   \item #LogInterval    -# Number of seconds between log output
  *   \item #DAQInterval    -# Number of seconds to run the monitor.  If not
  *     set, or equal to 0, magGlitch will not stop unless killed.
  *   \item #DecayTime      -# Coefficient of exponential decay for the running
  *     averages and standard deviations.
  *   \item #TriggerON      -# Flag whether or not to attempt to write triggers
  *     for each glitch.
  *   \item #Channel        -# Channel entry line.  All specs on the entry line
  *     must be separated by a single space.  The order of specs are:\\
  *     #<chan_name> <sample> <lo_thresh> <hi_thresh> <filter> <order> #
  *     #<lo_band> <hi_band> [<com>] End#\\
  *     where #chan_name# is the name of channel to be monitored, #sample# is
  *     the sampling rate of the channel, #lo/hi_thresh# are the two sets of
  *     thresholds to be used, #filter# can be one of {\em LowPass},
  *     {\em BandPass}, or {\em HighPass}, #order# is the order of Butterworth
  *     filter to be used, #lo/hi_band# are the frequency limits of the filter
  *     (only one should be specified when using {\em LowPass} or 
  *     {\em HighPass} filters, and #com# is an optional comment to be placed
  *     at the end of each data record for easy identification of glitches.
  * \end{itemize}
  *
  * {\large {\bf Suggested Uses}}\\
  * As one can see in the above configuration file, not all channels are set to
  * monitor the same bands.  The premise behind this was that to first order,
  * all axes have the same response to a generalized magnetic field transient.
  * Obviously this is not true, but in order to avoid the many dominating peaks
  * in the magnetometer power spectrum and still monitor a relatively wide
  * bandwidth, the magnetometer axes were interspersed between major spectral
  * peaks.  In the future, the author plans to allow multiple bands to be
  * monitored for each channel, which would allow axes' configurations to look
  * more symmetrical.
  * @memo Magnetic Field Glitch Monitor
  */
//@{

/** @name magGlitch internal constants and functions
 */
//@{
/// IIR-filtering kOffsetFactor seconds of time series 1.0 to 2.0 second
#define kOffsetFactor 3/2

/// wait time after end time of an event
const float kWaitTime = 0.1;

/// minimum time duration (second) of an event for low threshold
const float kMinEventDuration = 0.1;

/// maximum time interval (second) stop calculation of standard deviation
const float kMaxSDevSuspend = 300.0;

/// minimum density of data point above threshold during an event 
const float kMinDataPointDensity = 0.1;

/// histogram bin size
const float kHistBinSize = 0.2;

/// number of histogram bins
const int kHistBinNumber = 50;

/// changes an integer into a string
template<class T> std::string tostring(const T& n);

/// shifts a time series by amount dt (not functional yet)
bool shift(TSeries &ts, const Interval &dt);
//@}

/** @name magGlitch
 */
//@{
/** @memo Magnetic Field Glitch Monitor
  * @author Masahiro Ito, modified by Rauha Rahkola
  * @version 0.1; Last modified Nov 8, 2001
  */
class magGlitch : public DatEnv, TrigClient, MonServer
{

public:
  enum { kConfigTag = 0,
	 kConfigDataFilePath = 1,
	 kConfigDAQInterval = 1,
	 kConfigDecayTime = 1,
	 kConfigTriggerON = 1,
	 kConfigLogFilePath = 1,
	 kConfigLogInterval = 1,
	 kConfigChannelName = 1,
	 kConfigSampleFreq = 2,
	 kConfigThresholdLow = 3,
	 kConfigThresholdHi = 4,
	 kConfigFilterMode = 5,
	 kConfigFilterOrder = 6,
	 kConfigCutOffFreqLow = 7,
	 kConfigCutOffFreqHi = 8,
	 kConfigTrigComment = 9
  };

  // channel status infos
  std::vector<ChannelStatus> fCHStatus;

  /** Constructor for magGlitch
   */
  magGlitch(int,const char**);

  /** Destructor for magGlitch
   */
  ~magGlitch(void);

  /** Serve data to MonClient
   */
  void Attention(void);

  /** Process one second of data
   */
  void ProcessData(void);


  
private:
  // temporary buffer to keep 2 sec of time series
  float *fData;
  // double *fData;

  // temporary buffer to keep 1.25 sec of filtered time series
  float *fFilterData;
  // double *fFilterData;

  // max and min data in a event
  float fMaxData,fMinData;

  // number of channels to be motnitored
  int fNChannel;

  // monitor duration
  int fDAQInterval;

  // time interval for updating log file
  int fLogInterval;

  // weighed-average decaytime and decay constant
  float fDecayTime,fDecay;

  // total number of loops executed since monitor started
  long fCount;

  // end time of previous data
  long fLastEndTime;

  // 
  long fLogStartTime;

  // log and data file name
  std::string fLogName,fDataName;

  // file stream for log and data file
  std::ofstream fLogFile,fDataFile;

  // 0:no trigger to Trigger Manager, 1:send trigger
  bool fTriggerON;

  // read config file and set up channels
  void SetConfig(std::ifstream& config);

  // check filtered time series 
  void CheckEvent(ChannelStatus* ch);
  void CheckEvent(const float& data, const Time& currenttime, ChannelStatus* ch);

  // output event info to data file and send trigger
  void OutputEvent(ChannelStatus* ch);

  // output log file
  void OutputLog();

  // reset variables
  void Reset();

  // return mean value of time series
  const float GetMean(const ChannelStatus* ch) const;

  // return standard deviation of filtered time series
  //  const float GetSDev(const ChannelStatus* ch) const;
  const float GetSDev(const int sfreq) const;
  const float GetSDevF(const int sfreq) const;
  void UpdateSDevF(const float sdev, ChannelStatus* ch); 
};
//@}

#endif






