//
// Reference : Digital Filter Design (T.W. Parks & C.S. Burrus)
//
// 7/17/2002 
// Masahiro Ito
// Univerisity of Oregon
// masahiro@darkwing.uoregon.edu
//

#ifndef zp2zp_HH
#define zp2zp_HH

#include "FilterBase.hh"
#include "Complex.hh"
#include <string>

   const static double MACHEP =  1.11022302462515654042E-16;   /* 2**-56 */


	/** Convert a prototype low pass filter with 1 rad/sec cutoff to 
	  * low pass with cut off at w0 rad/sec.
	  * @memo Convert a prototype low pass filter to low pass with cut off at w0 rad/sec.
	  * @param wo Cut off frequency (rad/sec).
	  * @param nz Number of poles.
	  * @param zero Array for poles (rad/sec).
	  * @param np Number of poles.
	  * @param pole Array for poles (rad/sec).
	  * @param gain Filter gain.
	  */ 
   void lp2lp ( double w0, int& nz, 
              dComplex* zero, int& np, dComplex* pole, double& gain );

	/** Convert a prototype low pass filter with 1 rad/sec cutoff to 
	  * high pass with cut off at w0 rad/sec.
	  * @memo Convert a prototype low pass filter to low pass with cut off at w0 rad/sec.
	  * @param wo Cut off frequency (rad/sec).
	  * @param nz Number of poles.
	  * @param zero Array for poles (rad/sec).
	  * @param np Number of poles.
	  * @param pole Array for poles (rad/sec).
	  * @param gain Filter gain.
	  */ 
   void lp2hp ( double w0, int& nz, 
              dComplex* zero, int& np, dComplex* pole, double& gain );

	/** Convert a prototype low pass filter with 1 rad/sec cutoff to 
	  * band pass with center frequency at w0 rad/sec and band width, bw (rad/sec).
	  * @memo Convert a prototype low pass filter to band pass with center frequency at w0 rad/sec and band width, bw (rad/sec)..
	  * @param wo Center frequency (rad/sec).
	  * @param bw Band width
	  * @param nz Number of poles.
	  * @param zero Array for poles (rad/sec).
	  * @param np Number of poles.
	  * @param pole Array for poles (rad/sec).
	  * @param gain Filter gain.
	  */ 
   void lp2bp ( double w0, double bw, int& nz, 
              dComplex* zero, int& np, dComplex* pole, double& gain );

	/** Convert a prototype low pass filter with 1 rad/sec cutoff to 
	  * band stop with center frequency at w0 rad/sec and band width, bw (rad/sec).
	  * @memo Convert a prototype low pass filter to band stop with center frequency at w0 rad/sec and band width, bw (rad/sec)..
	  * @param wo Center frequency (rad/sec).
	  * @param bw Band width
	  * @param nz Number of poles.
	  * @param zero Array for poles (rad/sec).
	  * @param np Number of poles.
	  * @param pole Array for poles (rad/sec).
	  * @param gain Filter gain.
	  */
   void lp2bs ( double w0, double bw, int& nz, 
              dComplex* zero, int& np, dComplex* pole, double& gain );

	/** Convert prototype low pass filter to the specified filter type.
	  * @memo Convert prototype low pass filter to the specified filter type.
	  * @param type Filter type 0: Low Pass 2: High Pass 3: Band Pass 4:Band Stop
	  * @param f1 Pass band edge (Hz).
	  * @param f2 Another pass band edge (Hz).
	  * @param nz Number of poles.
	  * @param zero Array for poles (rad/sec).
	  * @param np Number of poles.
	  * @param pole Array for poles (rad/sec).
	  * @param gain Filter gain.
	  */
   bool zp2zp ( Filter_Type type, double f1, double f2,
              int& nz, dComplex* zero, int& np, dComplex * pole, double& gain );

	/** Convert filter type in char array to enumerated filter type integer.
	  * @memo Convert filter type in char array to enumerated filter type integer.
	  * @param type_name lowpass, highpass, bandpass or bandstop (case insensitive)
	  * @param type Filter type 0: Low Pass 2: High Pass 3: Band Pass 4:Band Stop.
	  * @return bool True if successful.
	  */
   bool getFilterType ( const char* type_name, Filter_Type& type );

	/** Convert filter type enumerated type to filter type string.
	  * @memo Convert filter type enumerated type to filter type string.
	  * @param type Filter type 0: Low Pass 2: High Pass 3: Band Pass 4:Band Stop.
	  * @return filter type.
	  */
   std::string getFilterString ( Filter_Type type );
#endif
